==> 20190330_203812.tar.gz: sys/arch/sparc64/include/userret.h <==
/*	$NetBSD: userret.h,v 1.9 2009/10/17 08:50:49 nakayama Exp $ */

/*
 * Copyright (c) 1996-2002 Eduardo Horvath.  All rights reserved.
 * Copyright (c) 1996
 *	The President and Fellows of Harvard College. All rights reserved.
 * Copyright (c) 1992, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * All advertising materials mentioning features or use of this software
 * must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory.
 *	This product includes software developed by Harvard University.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 *	This product includes software developed by Harvard University.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)trap.c	8.4 (Berkeley) 9/23/93
 */

#include <sys/userret.h>

static __inline void userret(struct lwp *, register_t,  u_quad_t);
/*
 * Define the code needed before returning to user mode, for
 * trap, mem_access_fault, and syscall.
 */
static __inline void
userret(struct lwp *l, register_t pc, u_quad_t oticks)
{

  /* FIXME pc is int?????????*/
#if 1
  printf("userret(%s, pc=%lx) entry\n", l->l_name, pc);
#endif  
	struct proc *p = l->l_proc;

	mi_userret(l);

	/*
	 * If profiling, charge recent system time to the trapped pc.
	 */
	if (p->p_stflag & PST_PROFIL)
		addupc_task(l, pc, (int)(p->p_sticks - oticks));
#if 1
	int nsaved = curpcb->pcb_nsaved;
	if (nsaved > 0) {
	  printf("userret() nsaved = %d\n", nsaved);
#if 1
	  if (nsaved ) {
	    for (int i=0; i<nsaved; i++) {
	      printf("rw index %d\n", i);
	      printf("locals:");
	      for (int j=0; j < 8; j++) {
		printf("  %%l%d  %016lx", j, (long unsigned int)curpcb->pcb_rw[i].rw_local[j]);
	      }
	      printf("\n");
	      printf("ins   :");
	      for (int j=0; j < 8; j++) {
		printf("  %%i%d  %016lx", j, (long unsigned int)curpcb->pcb_rw[i].rw_in[j]);
	      }
	      printf("\n");
	    }
	    if (nsaved > 6) {
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      printf("******************************is 6/7 valid???? **************\n");
	      /*Debugger();*/
	    }
	  }
#endif	  
	}
	if (nsaved < 0) {
	  Debugger();
	}
#endif	

	if (pc == 0x2be734 && nsaved == 1) {
	  printf("FIXME userret(%s, pc=%lx) nsaved %d  \n", l->l_name, pc, curpcb->pcb_nsaved);
	  curpcb->pcb_nsaved = 0;
	  printf("FIXME userret(%s, pc=%lx) nsaved %d \n", l->l_name, pc, curpcb->pcb_nsaved);
	}
#if 0
  printf("userret(%s, pc=%lx) exit\n", l->l_name, pc);
#endif
	
}

static __inline void share_fpu(struct lwp *, struct trapframe64 *);
/*
 * If someone stole the FPU while we were away, do not enable it
 * on return.  This is not done in userret() above as it must follow
 * the ktrsysret() in syscall().  Actually, it is likely that the
 * ktrsysret should occur before the call to userret.
 *
 * Oh, and don't touch the FPU bit if we're returning to the kernel.
 */
static __inline void
share_fpu(struct lwp *l, struct trapframe64 *tf)
{
	if (!(tf->tf_tstate & TSTATE_PRIV) && fplwp != l)
		tf->tf_tstate &= ~TSTATE_PEF;
}

==> 20201103_213646.tar.gz: sys/arch/sparc64/include/userret.h <==
/*	$NetBSD: userret.h,v 1.9 2009/10/17 08:50:49 nakayama Exp $ */

/*
 * Copyright (c) 1996-2002 Eduardo Horvath.  All rights reserved.
 * Copyright (c) 1996
 *	The President and Fellows of Harvard College. All rights reserved.
 * Copyright (c) 1992, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * All advertising materials mentioning features or use of this software
 * must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Lawrence Berkeley Laboratory.
 *	This product includes software developed by Harvard University.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 *	This product includes software developed by Harvard University.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)trap.c	8.4 (Berkeley) 9/23/93
 */

#include <sys/userret.h>

static __inline void userret(struct lwp *, int,  u_quad_t);
/*
 * Define the code needed before returning to user mode, for
 * trap, mem_access_fault, and syscall.
 */
static __inline void
userret(struct lwp *l, int pc, u_quad_t oticks)
{
#if 0
  printf("userret() pc = %x\n", pc);
#endif
#if 0
  if (curpcb->pcb_nsaved) {
    for (int iii = 0; iii < curpcb->pcb_nsaved+1; iii++) {
      printf("saved register windows (%d) offset %d", curpcb->pcb_nsaved, iii);
      printf("\nlocals:");
      for (int iiii = 0; iiii < 8; iiii++) {
	if ( iiii % 4 == 0)
	  printf("\n");
	printf("%016lx ", curpcb->pcb_rw[iii].rw_local[iiii]);
      }
      printf("\nins:");
      for (int iiii = 0; iiii < 8; iiii++) {
	if ( iiii % 4 == 0)
	  printf("\n");
	printf("%016lx ", curpcb->pcb_rw[iii].rw_in[iiii]);
      }
      printf("\n");
    }
    printf("\n");
  }
#endif  
	struct proc *p = l->l_proc;

	mi_userret(l);

	/*
	 * If profiling, charge recent system time to the trapped pc.
	 */
	if (p->p_stflag & PST_PROFIL)
		addupc_task(l, pc, (int)(p->p_sticks - oticks));
}

static __inline void share_fpu(struct lwp *, struct trapframe64 *);
/*
 * If someone stole the FPU while we were away, do not enable it
 * on return.  This is not done in userret() above as it must follow
 * the ktrsysret() in syscall().  Actually, it is likely that the
 * ktrsysret should occur before the call to userret.
 *
 * Oh, and don't touch the FPU bit if we're returning to the kernel.
 */
static __inline void
share_fpu(struct lwp *l, struct trapframe64 *tf)
{
	if (!(tf->tf_tstate & TSTATE_PRIV) && fplwp != l)
		tf->tf_tstate &= ~TSTATE_PEF;
}
